#!/usr/bin/env perlml
##################################################################################################
#
# DB actions script to support statusviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

use CGI;
use CGI::Carp qw(fatalsToBrowser);

use utils qw(logit);
use DBI;

use Config::Simple;

#read in a simple config file to get the db access parameters
my $cfg         = new Config::Simple('../../configs/statusviewer.conf');
my $db_source   = $cfg->param('db_source');
my $db_user     = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug       = $cfg->param('debug') || 0;
my $db_switch   = $cfg->param('db_switch');

my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

&logit(qq{\n\nprocessing parameters:}) if $debug;
while (($holder,$value)=each (%params)) {
   &logit(qq{   $holder = $value}) if $debug;
}

my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $db_source: $DBI::errstr";

#build a structure so I can look up the id of the item type 
my %typemap;
$sql = qq{SELECT
    status_types.name,
    status_types.id
FROM
    status_types};
    
&logit(qq{status sql ($sql)}) if $debug;
$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$typename,\$typeid);

while ($sth->fetch()) {
   $typemap{$typename}=$typeid;
}

#build a structure so I can look up the id of the status when it isnt modified on the row
my %statusmap;
$sql = qq{SELECT
    status_states.name,
    status_states.id
FROM
    status_states};
&logit(qq{status sql ($sql)}) if $debug;
$sth = $dbh->prepare($sql);
$sth->execute();
$sth->bind_columns(\$status,\$id);

while ($sth->fetch()) {
   $statusmap{$status}=$id;
}


my @updaterows = split(',',$params{ids});
&logit(qq{ updaterows   @updaterows}) if $debug;

my $resultString = qq{<data>};

#0 -> id
#1 -> year
#2 -> week
#3 -> item
#4 -> description
#5 -> weight
#6 -> status
#7 -> delete checkbox

foreach $rowid (@updaterows) {

   my $itemidkey = qq{$rowid\_c0};
   my $itemid = $params{$itemidkey};
   &logit(qq{ itemidkey $itemid}) if $debug;

   my $weekkey = qq{$rowid\_c1};
   my $week = $params{$weekkey};
   &logit(qq{ week $week}) if $debug;

   my $typeidkey = qq{$rowid\_c2};
   my $typeid = $params{$typeidkey};
   if ($typeid !~ /^\d+$/) {
      $typeid = $typemap{$typeid};
   }
   
   &logit(qq{ week $week}) if $debug;
   
   my $itemtxtkey = qq{$rowid\_c3};
   my $itemtxt = $params{$itemtxtkey};
   &logit(qq{ itemtxt $itemtxt}) if $debug;
   #remove some chars that might break the sql from inserting
   $itemtxt =~ s/\'//g;
   $itemtxt =~ s/\,//g;
   
   my $statusidkey = qq{$rowid\_c4};
   my $statusid = $params{$statusidkey};
   if ($statusid !~ /^\d+$/) {
      $statusid = $statusmap{$statusid};
   }
   &logit(qq{ statusid $statusid}) if $debug;

   my $deletekey = qq{$rowid\_c5};
   my $delkey = $params{$deletekey};
   &logit(qq{ deletekey $deletekey}) if $debug;

   #!nativeeditor_status = inserted 
   my $rowstatusidkey = qq{$rowid\_!nativeeditor_status}; 
   my $rowstatuskey = $params{$rowstatusidkey};
   &logit(qq{ rowstatuskey $rowstatuskey}) if $debug;

   if ($rowstatuskey eq "inserted") {
      &logit(qq{ new record to insert}) if $debug;
      # id year name weight description receiver status modified
      my $sql = qq{insert into status_items(week,item,typeid,status) values ('$week','$itemtxt',$typeid,$statusid)};
      &logit(qq{ insert sql - $sql}) if $debug;
      $dbh->do($sql);
      $resultString .= qq{\n<action type='inserted' sid='$rowid' tid='$rowid'/>};
   }elsif ($rowstatuskey eq "updated") {
      if ($delkey == 1) {
         my $sql = qq{delete from status_items where id= $itemid};
         &logit(qq{ delete sql - $sql}) if $debug;
         $dbh->do($sql);
         $resultString .= qq{\n<action type='deleted' sid='$rowid' tid='$rowid'/>};
      }else{
         my $sql = qq{update status_items set week='$week',item='$itemtxt',typeid=$typeid,status=$statusid where id = $itemid}; 
         &logit(qq{ update sql - $sql}) if $debug;
         $dbh->do($sql);
         $resultString .= qq{\n<action type='updated' sid='$rowid' tid='$rowid'/>};
      }
   }#elsif
};

$resultString .= qq{\n</data>};

&logit(qq{$resultString}) if $debug;


print "Content-type: text/xml", "\n\n";
print qq{<?xml version="1.0"  encoding="ISO-8859-1"?>};

print qq{
$resultString
};
