#!/usr/bin/env perlml
##################################################################################################
#
# search actions script to support statusviewer.cgi
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#
$|=1;    # set output buffering to off
$,=" ";  # set output field separator

#use xoutils qw(logit dbconnect_lookup);
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;
use Date::Calc qw(:all);

use utils qw(logit);

#read in a simple config file to get the db access parameters
my $cfg         = new Config::Simple('../../configs/statusviewer.conf');
my $db_source   = $cfg->param('db_source');
my $db_user     = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug       = $cfg->param('debug') || 0;
my $db_switch   = $cfg->param('db_switch');

&logit(qq{(db_source,db_user,db_password ($db_source,$db_user,$db_password)}) if $debug;

my $q = new CGI;
#pull the params into a hash
my %params = $q->Vars;

&logit(qq{\n\nprocessing parameters:}) if $debug;
while (($holder,$value)=each (%params)) {
   &logit(qq{   $holder = $value}) if $debug;
}

my $dbh = DBI->connect($db_source,$db_user,$db_password)or die "Can't connect to $db_source: $DBI::errstr";
	
#build the weeks menu after a change is made to the month or year in the add row form
if ($params{action} =~ /fetchtotals/i) {

	#build a structure so I can look up the id of the status and color when it isnt modified on the row
	my %statusmap;
	my %colormap;
	my $sql = qq{SELECT
		status_states.name,
		status_states.id,
		status_states.color
	FROM
		status_states
	order by status_states.id};

	$sth = $dbh->prepare($sql);
	$sth->execute();
	$sth->bind_columns(\$status,\$id,\$color);

	while ($sth->fetch()) {
	   $statusmap{$id}=$status;
	   $colormap{$id}=$color;
	}

	#=========================================================
	#fetch the totals from the db
	my $sql = qq{
	SELECT
		count(*),
		status_items.status,
		status_items.typeid,
		status_types.name,
		upper(status_states.name),
		status_states.color
	FROM
		status_items
	INNER JOIN
		status_states
	ON
		(
			status_items.status = status_states.id)
	INNER JOIN
		status_types
	ON
		(
			status_items.typeid = status_types.id)
	WHERE 1 = 1
	};

	$sql .= qq{\n and status_items.typeid = $params{typeid}} if ($params{typeid});
	$sql .= qq{\n and status_items.name like '\%$params{likestring}\%'} if ($params{likestring});


	my $yearstring     = $params{year} || "-";
	my $monthstring    = $params{month} || "-";
	my $daysbackstring = $params{daysback} || "-";
		
	if ($params{daysback}) {
		$sql .= qq{\n and status_items.week > date_sub(curdate(), interval $params{daysback} day)};
	} else{
		$sql .= qq{\n and status_items.week like '$params{year}\%'} if ($params{year});
		$sql .= qq{\n and status_items.week like '\%-$params{month}-\%'} if ($params{month});
		$sql .= qq{\n and week(status_items.week)  = ($params{week}-1)} if ($params{week});
	}

	if ($params{statusid}) {
	   if ($params{statusid} == 999) {
		  #the 'all but acquired' is selected
		  $sql .= qq{\n and status_items.status != 3};
	   }else{
		  $sql .= qq{\n and status_items.status = $params{statusid}};
	   }
	}

	$sql .= qq{\n group by upper(status_items.status),status_items.typeid};

	&logit(qq{sql:\n$sql}) if $debug;

	$sth = $dbh->prepare($sql);
	$sth->execute();
	$sth->bind_columns(\$count,\$statusid,\$typeid,\$typename,\$statesname,\$statuscolor);

	while ($sth->fetch()) {
	   $statushash{"$statusid"} += $count;
	   $itemscount += $count;
	}

	#===================================================
	#build the status line and send it out

	#$statusline = qq{Days Back: $daysbackstring Year : $yearstring&nbsp;&nbsp;&nbsp;&nbsp;Month : $monthstring&nbsp;&nbsp;&nbsp;&nbsp;Items : $itemscount&nbsp;&nbsp;&nbsp;&nbsp;-};

	my $statusline;
	my @statuslist = (sort keys %statusmap);
	for ($t=0; $t<=scalar(@statuslist); $t++) {
	   my $statusname = $statusmap{$t};
	   my $statuscolor = $colormap{$t};
	   next if ($statusname =~ /^ *$/);
	   my $value = int($statushash{$t});
	   $statusline .= qq{&nbsp;&nbsp;&nbsp;&nbsp;<span style="color:$statuscolor">$statusname : $value</span>};
	}

	#return the results
	print qq{
	$statusline
	};
	exit 1;
}#if fetchtotals

#build the weeks menu after a change is made to the month or year in the add row form
if ($params{action} =~ /weeksearch/i) {
   &logit(qq{WEEKSEARCH}) if $debug;
   my $year   = $params{year};
   my $month  = $params{month};
   my $numwks = Weeks_in_Year($year);
   #$numwks++;
   &logit(qq{numwks ($numwks)  year ($year)  month ($month)}) if $debug;
   my @weekslist;
   my ($week,$y) = Week_of_Year($year,$month,1);
   &logit(qq{week year ($week,$y) }) if $debug;
   for (1..5) {
      if ($week >$numwks) {
         $week=1;
         $year++;
      }
      my ($mon_year, $mon_month, $mon_day) = Monday_of_Week($week, $year);
      $mon_month = sprintf('%02d',$mon_month);
      $mon_day   = sprintf('%02d',$mon_day);
      $week      = sprintf('%02d',$week);
      push(@weekslist,"$week:$mon_year-$mon_month-$mon_day");
      $week++;
   }

   my $weekslist = join(',',@weekslist);
   $weekslist =~ s/\n//g;
   $weekslist =~ s/\r//g;

   &logit(qq{weekslist: $weekslist}) if $debug;
   print "Content-type: text/plain", "\n\n";
   print qq{$weekslist};
   exit 1;
}#if weeksearch

#-----------------------------------------------------------------------------------------------------
#search for Items
&logit(qq{   doing dbsearch()}) if $debug;

my $sql = qq{
SELECT
    status_items.id,
    status_items.week,
    status_items.item,
    status_types.name,
    status_states.name,
    status_states.color
FROM
    status_items
INNER JOIN
    status_states
ON
    (
        status_items.status = status_states.id)
INNER JOIN
    status_types
ON
    (
        status_items.typeid = status_types.id)
WHERE 1 = 1
};

if ($params{daysback}) {
	$sql .= qq{\n and status_items.week > date_sub(curdate(), interval $params{daysback} day)};
} else{
	$sql .= qq{\n and status_items.week like '$params{year}\%'} if ($params{year});
	$sql .= qq{\n and status_items.week like '\%-$params{month}-\%'} if ($params{month});
	$sql .= qq{\n and week(status_items.week)  = ($params{week}-1)} if ($params{week});
}
$sql .= qq{\n and status_items.typeid = $params{typeid}} if ($params{typeid});

# $sql .= qq{\n and status_states.id = '$params{statusid}'} if ($params{statusid});
if ($params{statusid}) {
   if ($params{statusid} == 999) {
      #the 'all but acquired' is selected
      $sql .= qq{\n and status_states.id != 3};
   }else{
      $sql .= qq{\n and status_states.id = $params{statusid}};
   }
}
$sql .= qq{\n and status_items.item like '\%$params{likestring}\%'} if ($params{likestring});

#$sql .= qq{\n order by status_items.modified DESC,status_types.name ASC};
#$sql .= qq{\n order by status_items.week DESC,status_types.name ASC};
$sql .= qq{\n order by status_items.week DESC,status_items.modified DESC};

&logit(qq{sql:\n$sql}) if $debug;

my $resultString;

$sth = $dbh->prepare($sql);
$sth->execute();

my $rowcount=0;
my $grandtotal=0;
$sth->bind_columns(\$itemid,\$week,\$item,\$itemtype,\$status,\$statuscolor);
my $resultString;
while ($sth->fetch()) {
$resultString .= qq{\n<row id='$rowcount'>
<cell>$itemid</cell>
<cell>$week</cell>
<cell>$itemtype</cell>
<cell>$item</cell>
<cell style="color:$statuscolor">$status</cell>
<cell>0</cell>
</row>};
$rowcount++;
$grandtotal += $itemweight;
}

$resultString =~ s/\&/\&amp;/g;
$resultString =~ s/\n//g;
$resultString =~ s/\r//g;

#&logit(qq{\n\nresults:\n$resultString}) if $debug;

print "Content-type: text/xml", "\n\n";
print qq{<?xml version="1.0"  encoding="ISO-8859-1"?><rows total_count='$rowcount' pos='0'>$resultString</rows>};
#print q{<?xml version="1.0" encoding="ISO-8859-1"?><rows><row id='0'><cell>AMA</cell><cell>5BA</cell><cell>2011-12-27 15:49:19</cell><cell>1439.965000</cell><cell></cell><cell>1439.96500</cell><cell>4107886300</cell><cell></cell><cell></cell><cell>4435246245</cell><cell>0000</cell><cell></cell><cell>0</cell><cell>1</cell><cell>4435240000</cell><cell></cell><cell></cell><cell>2010-11-13 00:30:45</cell><cell></cell></row></rows>};
