#!/usr/bin/env perlml
##################################################################################################
#
# Frontend for the status database
#
##################################################################################################
## +++++++++++++++++++++++++++ Maintenance Log +++++++++++++++++++++++++++++
# Monty Scroggins   Script created.
#

$|=1;    # set output buffering to off
$,=" ";  # set output field separator

use CGI;
use CGI::Carp qw(fatalsToBrowser);
use DBI;
use Config::Simple;
use Date::Calc qw(:all);

use utils qw(logit);

#read in a simple config file to get the db access parameters
my $cfg         = new Config::Simple('../../configs/statusviewer.conf');
my $db_source   = $cfg->param('db_source');
my $db_user     = $cfg->param('db_user');
my $db_password = $cfg->param('db_password');
my $debug       = $cfg->param('debug') || 0;
my $db_switch   = $cfg->param('db_switch');

&logit(qq{(db_source,db_user,db_password ($db_source,$db_user,$db_password)}) if $debug;

#establish a connection to the db
my $dbh = DBI->connect($db_source,$db_user,$db_password) || die "Can't connect: $DBI::errstr\n";

#-----------------------------------------------------------------------------------------------------
# Days option menu  - fixed menu
my $daysoptionmenuguts = qq{
<option value="10" Selected>10</option>
<option value="15">15</option>
<option value="30">30</option>
<option value="60">60</option>
<option value="120">120</option>
<option value="180">180</option>
<option value="365">365</option>
</select>
};


#-----------------------------------------------------------------------------------------------------
#year option menu for the search form (for the range of the data in the database)
my ($currday,$currmonth,$curryear)=(localtime(time()))[3,4,5];
#make the month one relative
$currmonth++;
#make the year a real year
my $fullcurryear = ($curryear + 1900);
my $nextfullyear = ($fullcurryear + 1);
my $padcurrmonth = sprintf('%02d',$currmonth);

#setup some month and year values for the search form..  It needs to default to the year/month of the current status week
#calculate the monday of the current week so the date option menus can be set to indicate the current status week period
my ($weekofyear,$statusyear) = Week_of_Year($curryear,$currmonth,$currday);
my ($selectstatusyear,$selectstatusmonth,$selectstatusday) = Monday_of_Week($weekofyear,$statusyear);
$selectstatusyear += 1900;
    
my $yearoptionmenuguts; #for the search form

#get the range of years of the data in the database and make sure the current year is in there as well 
#after the year rolls over, there is no data for current year and therefore no data can be entered
my $yearsql = qq{
select year from (
SELECT
    substr(week,1,4) as year
FROM
    status_items
union
select date_format(curdate(),'%Y') as year
) a
group by a.year
ORDER BY
    a.year desc
};

my $yearval;
$sth = $dbh->prepare($yearsql);
$sth->execute();
$sth->bind_columns(\$yearval);
while ( $sth->fetch() ) {
	my $yearseloption='';
    $yearoptionmenuguts .= qq{<option value="$yearval" $yearseloption>$yearval</option>\n};
}
$sth->finish();
$yearoptionmenuguts .= qq{</select>};


#-----------------------------------------------------------------------------------------------------
# MONTH option menu for search form

my $monthoptionmenuguts; #for the search form

my @months = qw(
nada
January
February
March
April
May
June
July
August
September
October
November
December
);

for ($t=1; $t<=12; $t++) {
	my $monthseloption = ""; 
	my $longmonthnum = sprintf('%02d',$t);
	$monthoptionmenuguts .= qq{<option value="$longmonthnum" $monthseloption>$months[$t]</option>\n};
}

$monthoptionmenuguts .= qq{</select>};
#-----------------------------------------------------------------------------------------------------
# initial MONTH option menu for add entry - gets replaced if the add year or month is changed

my $weekoptionmenuguts;
#get the week number of the current day
my ($currweeknum) = (Week_of_Year($fullcurryear,$currmonth,$currday))[0];
my $currweekstring;
#get the week number of the first day of the month
my ($week,$varyear) = Week_of_Year($fullcurryear,$currmonth,1);
#get the total number of weeks in the month
my $numwks = Weeks_in_Year($fullcurryear);
&logit(qq{week $varyear ($week,$varyear) }) if $debug;
for (1..5) {
   if ($week >$numwks) {
	  $week=1;
	  $varyear++;
   }
   my $weekseloption = "";   
   my ($mon_year, $mon_month, $mon_day) = Monday_of_Week($week, $varyear);
   $mon_month = sprintf('%02d',$mon_month);
   $mon_day   = sprintf('%02d',$mon_day);
	if ($week == $currweeknum) {
		$weekseloption = qq{selected};
	}  
   $weekoptionmenuguts .= qq{<option value="$week" >$mon_year-$mon_month-$mon_day</option>\n};
   push(@weekslist,"$weeknum:$mon_year-$mon_month-$mon_day");
   $week++;
}
$weekoptionmenuguts .= qq{</select>};

#-----------------------------------------------------------------------------------------------------
# type option menu
my $typeoptionpopulate;
my $typeoptionmenuguts;
my $typesql = qq{
SELECT
    id,
    name
FROM
    status_types
ORDER BY
    disp_order,name
};

$sth = $dbh->prepare($typesql);
$sth->execute();
$sth->bind_columns(\$val,\$label);
while ( $sth->fetch() ) {
    $typeoptionmenuguts .= qq{<option value="$val">$label</option>\n};
    $typeoptionpopulate .= qq{\ntypecombo.put('$val','$label');};
}
$typeoptionmenuguts .= qq{</select>};
$sth->finish();

#-----------------------------------------------------------------------------------------------------
# status option menu
my $statusoptionpopulate; 
my $statusoptionmenuguts; 
my $statsql = qq{
SELECT
    id,
    name,
	default_select
FROM
    status_states
ORDER BY
    id ASC
};

$sth = $dbh->prepare($statsql);
$sth->execute();
$sth->bind_columns(\$val,\$label,\$defsel);
while ( $sth->fetch() ) {
	my $selectswitch = ""; 
	if ($defsel == 1) {
		$selectswitch = qq{selected};
	}
	$statusoptionmenuguts .= qq{<option value="$val" $selectswitch>$label</option>\n};
	$statusoptionpopulate .= qq{\nstatcombo.put('$val','$label');};
}
$statusoptionmenuguts .= qq{</select>};
$sth->finish();


 #    #  ######  #####   #####     ##     ####   ###### 
 #    #  #       #    #  #    #   #  #   #    #  #
 #    #  #####   #####   #    #  #    #  #       #####
 # ## #  #       #    #  #####   ######  #  ###  #
 ##  ##  #       #    #  #       #    #  #    #  #
 #    #  ######  #####   #       #    #   ####   ######

print "Content-type: text/html", "\n\n";

print qq{
<html>
<head>
<script src="/js/dhtmlxsuite/codebase/dhtmlx.js"></script>
<link rel="stylesheet" type="text/css" href="/js/dhtmlxsuite/skins/skyblue/dhtmlx.css"/>
<link rel="stylesheet" type="text/css" href="/css/statusviewer.css">
<script src="/js/prototype.js"></script>

<title>Status Item Manager</title>
<script language=Javascript>
function showMessage(message) {
   var span = document.getElementById("centermessagespan");
   span.innerHTML = message;
}

function setDefaults() {
   document.getElementById("typeid").value = '';
   document.getElementById("statusid").value = '';
   document.getElementById("daysback").value = 10;
   disableDateRange();
   document.getElementById("likestring").value = '';
   showMessage('');
}

function reloadGrid() {
   var daysback = document.getElementById("daysback").value;
   var typeid = document.getElementById("typeid").value;
   var statusid = document.getElementById("statusid").value;
   var likestring = document.getElementById("likestring").value;
   var year = document.getElementById("year").value;
   var month = document.getElementById("month").value;
   var week = document.getElementById("week").value;
   //alert('week '+week);
   var params = 'daysback='+daysback+'&typeid='+typeid+'&statusid='+statusid+'&likestring='+likestring+'&year='+year+'&month='+month+'&week='+week+'&action=search';
   //alert(params);
   mygrid.clearAll();

   var url = "status_search.cgi";

   mygrid.load(url+"?"+params,
   function(){
   //display the counts message on the left
   document.getElementById("leftmsgspan").style.visibility='visible';
   //display the export message on the right
   document.getElementById("rightmsgspan").style.visibility='visible';
   //display the record count on the left
   document.getElementById("leftmsgspan").innerHTML=mygrid.getRowsNum()+' Items';
   },
   'xml');
   fetchtotals();
}

function loadInitialGrid() {
   //alert('loadinitialgrid ');
   disableDateRange();
   
   //default to only going back the number of days set by default in the daysback option.  No more matching up dates and weeks
   var daysback=document.getElementById("daysback").value;
   var params = 'daysback='+daysback+'&action=search';

   //alert(params);
   mygrid.clearAll();

   var url = "status_search.cgi";

   mygrid.load(url+"?"+params,
   function(){
   //display the counts message on the left
   document.getElementById("leftmsgspan").style.visibility='visible';
   //display the export message on the right
   document.getElementById("rightmsgspan").style.visibility='visible';
   //display the record count on the left
   document.getElementById("leftmsgspan").innerHTML=mygrid.getRowsNum()+' Items';
   },
   'xml');
   fetchtotals();
   //set the search week option to be 'All'
   document.getElementById("week").value = '';
   //set the add row week option to be the current week
   document.getElementById("addweek").value = "$currweeknum";
   //set the add row status option menu to 'Completed'
   document.getElementById("addstatusid").value = 3;
}

//populate the list of weeks based on the selection parameters etc
function fetchWeeksList() {
   var year  = document.getElementById("addyear").value;
   var month = document.getElementById("addmonth").value;
   var params='year='+year+'&month='+month+'&action=weeksearch';
   //alert('params '+params);
   var url = "status_search.cgi";
   var myAjax = new Ajax.Request(url,{method: 'post', parameters: params,onSuccess: popWeeksList})
}

function popWeeksList(originalRequest) {
    var returndata=originalRequest.responseText;
    //alert('return '+returndata);
    var weeksArray=returndata.split(",");
    var ddl = document.getElementById('addweek');
    ddl.options.length = 0;
    ddl.options[0] = new Option ('Select', '');
    for(i=0;i<weeksArray.length;i++) {
		var weekString = weeksArray[i];
		//remove the count from the string for the value
		var idxval  = weekString.indexOf(':');
		var weeknum = weekString.substring(0,idxval);
		var weekval = weekString.substring(idxval+1,13);
		ddl.options[i+1] = new Option (weekval,weeknum);
    }
}

//populate the list of weeks based on the selection of the year
function fetchSearchWeeksList() {
   var year  = document.getElementById("year").value;
   var month = document.getElementById("month").value;
   var params='year='+year+'&month='+month+'&action=weeksearch';
   //alert('params '+params);
   var url = "status_search.cgi";
   var myAjax = new Ajax.Request(url,{method: 'post', parameters: params,onSuccess: popSearchWeeksList})
}

function disableDaysBack(){
   document.getElementById("daysback").value='';
}

function disableDateRange(){
   document.getElementById("year").value  = 'All'
   document.getElementById("month").value = 'All'
   document.getElementById("week").value  = 'All'
}

function popSearchWeeksList(originalRequest) {
    var returndata=originalRequest.responseText;
    //alert('return '+returndata);
    var weeksArray=returndata.split(",");
    var ddl = document.getElementById('week');
    ddl.options.length = 0;
    ddl.options[0] = new Option ('All', '');
    for(i=0;i<weeksArray.length;i++) {
		var weekString = weeksArray[i];
		//remove the count from the string for the value
		var idxval  = weekString.indexOf(':');
		var weeknum = weekString.substring(0,idxval);
		var weekval = weekString.substring(idxval+1,13);
		//alert('idxval '+idxval+'  weeknum '+weeknum+'  weekval '+weekval);
		ddl.options[i+1] = new Option (weekval,weeknum);
    }
}

function fetchtotals(params) {
   var url = "status_search.cgi";
   var typeid = document.getElementById("typeid").value;
   var statusid = document.getElementById("statusid").value;
   var likestring = document.getElementById("likestring").value;
   var daysback=document.getElementById("daysback").value;
   var year=document.getElementById("year").value;
   var month=document.getElementById("month").value;
   var week = document.getElementById("week").value;
   var params='daysback='+daysback+'&typeid='+typeid+'&statusid='+statusid+'&likestring='+likestring+'&year='+year+'&month='+month+'&week='+week+'&action=fetchtotals';
   //alert(params);
   var myAjax = new Ajax.Request(url,{method: 'post', parameters: params,onComplete: showtotals});
}

function addRow() {
   //var addweek = document.getElementById("addweek").value; //doesnt work if repopulated by changing year select.  use selectedindex instead
   var weeksel = document.getElementById("addweek");
   var addweek = weeksel.options[ weeksel.selectedIndex ].text
   
   var addtypeid = document.getElementById("addtypeid").value;
   var additem = document.getElementById("additem").value;
   if (additem == "" || additem == null || additem.charAt(0) == ' '){
      alert('Item is a mandatory field.  \\n\\nPlease amend and retry.');
      return false;
   }
   //cant decide if its better to replace commas with the html equiv.   Decided just to remove them 
   //additem=additem.replace(/,/g, "&#44;");
   additem=additem.replace(/,/g, " ");
   var addstatusid = document.getElementById("addstatusid").value;
   var addstring = ','+addweek+','+addtypeid+','+additem+','+addstatusid
   //alert(addstring);
   var rowcount = mygrid.getRowsNum();
   mygrid.addRow(rowcount + 1, addstring);
}

function IsNumeric(input) {
    return (input - 0) == input && input.length > 0;
}

function showtotals(originalRequest) {
   var returndata=originalRequest.responseText;
   document.getElementById("totaltablecontainer").innerHTML=returndata;
}
</script>
</head>
<body onload="loadInitialGrid();">
<center>
<h3>Status Manager</h3>
  <table class="searchformtable">
     <tr>
     <th>Days Back</th>
     <th></th>
     <th>Year</th>
	 <th>Month</th>
	 <th>Week</th>
     <th>Type</th>
     <th>Status</th>
     <th>Containing</th>
     </tr>
	 <tr>
	  <td  class="toprowtdl">
            <select id="daysback" onChange=disableDateRange();>
            <option value=""></option>
            $daysoptionmenuguts
	  </td>
	  <td>-or-</td>
	  <td class="toprowtdl">
            <select id="year" onChange="disableDaysBack();fetchSearchWeeksList();">
            <option value=""></option>
            $yearoptionmenuguts
	  </td>
	  <td  class="toprowtdl">
            <select id="month" onChange="disableDaysBack();fetchSearchWeeksList();">
            <option value=""></option>
            $monthoptionmenuguts
	  </td>
	  <td  class="toprowtdl">
            <select id="week" onChange=disableDaysBack();>
            <option value=""></option>
            $weekoptionmenuguts
	  </td>
	  <td  class="toprowtdl">
            <select id="typeid">
            <option value="" selected>All</option> 
            $typeoptionmenuguts
            </select>
	  </td>
	  <td  class="toprowtdl">
            <select id="statusid">
            <option value="" selected>All</option>
            <option value="999">All But Completed</option>
            $statusoptionmenuguts
	  </td>
	  <td  class="toprowtdl">
            <input type="text" id="likestring"  onkeypress="{if (event.keyCode==13) return reloadGrid();}"></input>
	  </td>

	 </tr>
</table>

<!-- <span id="searchfooter"><span style="color:red;">*</span> = Recommended Minimum Search Fields</span> -->

<input type="button" class="formbutton" onclick="return reloadGrid();" value="Search"></input>
<input type="button" class="formbutton" onclick="setDefaults();" value="Reset"></input>
<!-- <input type="button" class="formbutton" name="" value="Export" onclick="mygrid.toExcel('/server/grid2excel/generate.php');"> -->
<br>
<div id="msgcontainer">
   <span id="centermessagespan"></span>
   <span id="rightmsgspan">
   <!-- <a href= 'javascript:mygrid.toPDF("https://dhtmlxgrid.appspot.com/export/pdf");'>PDF</a>&nbsp;&nbsp; -->
   <a href= "javascript:mygrid.toExcel('http://dhtmlxgrid.appspot.com/export/excel');">Export to Excel</a>
   </span>
   <span id="leftmsgspan">&nbsp;</span>
   <a href="javascript:void();" onclick="myDataProcessor.sendData();">Save Changes</a>
</div>
<div id="gridbox" style="width:1000px;height:500px;background-color:transparent;"></div>
<script>
mygrid = new dhtmlXGridObject('gridbox');
mygrid.setImagePath("/js/dhtmlxsuite/codebase/imgs/");
mygrid.setHeader(",Week,Type,Title,Status,Delete",
null,
["text-align:center;","text-align:center;","text-align:center","text-align:center;","text-align:center;","text-align:center"]
);
mygrid.setInitWidths("0,84,84,*,100,50");
mygrid.setColAlign("left,center,left,left,left,center");
mygrid.setColTypes("ro,ed,coro,txt,coro,ch")
mygrid.setColSorting("int,int,str,str,str,int");
//the actual skin is set by the includes at the top of the webpage.  this just sets the css strings to load
mygrid.setSkin("dhx_skyblue");
mygrid.init();

//mygrid.attachEvent("onRowSelect", function(id,ind){fetchDetails(id,ind);});
//set the combobox values for the year selection
var yearcombo = mygrid.getCombo(1); // index of the column to get combo object (zero relative)
//set the combobox values for the type selection
var typecombo = mygrid.getCombo(2); // index of the column to get combo object (zero relative)
$typeoptionpopulate
//set the combobox values for the status selection
var statcombo = mygrid.getCombo(4); // index of the column to get combo object (zero relative)
$statusoptionpopulate

myDataProcessor = new dataProcessor("./status_dbupdateall.cgi");
//lock feed url;
myDataProcessor.setTransactionMode("POST", true);
//set mode as send-all-by-post;
myDataProcessor.setUpdateMode("off");

myDataProcessor.init(mygrid);

//link dataprocessor to the grid;
myDataProcessor.attachEvent("onAfterUpdate", function() {
	//alert('onAfterUpdate completed');
	reloadGrid();
    fetchtotals();
	return true;
});
</script>
<br>
<div id="totaltablecontainer">&nbsp;</div>
<br>
<table id="addrowtable">
<tr>
    <th>Year</th>
	<th>Month</th>
	<th>Week</th>
    <th>Type</th>
    <th>Item</th>
    <th>Status</th>
</tr>
<tr>
<td>
    <select id="addyear" onChange=fetchWeeksList();>
    $yearoptionmenuguts
</td>
<td>
    <select id="addmonth" onChange=fetchWeeksList();>
    $monthoptionmenuguts
</td>
<td>
    <select id="addweek">
    $weekoptionmenuguts
</td>
<td>
    <select id="addtypeid">
    $typeoptionmenuguts
</td>
<td><input type="text" id="additem"</input></td>
<td>
    <select id="addstatusid">
    $statusoptionmenuguts
</td>
</tr>
</table>
<a href="javascript:void();" onclick="addRow();">Add Status Item</a>
<br>
</body>
</html>
};

