package utils;

use DBI;

use vars qw(
   @ISA @EXPORT
   @EXPORT_OK
   %EXPORT_TAGS
   $dbh
   );

use Exporter;

#@ISA = qw(Exporter $sendto);
@ISA = qw(Exporter);

@EXPORT = qw();
@EXPORT_OK = qw(
  curr_date
  find_date
  unix_date
  mail_message
  logit
  );

%EXPORT_TAGS = (all => [@EXPORT_OK] );

#return a date offset from a specified UNIX date in a specified format
sub find_date {
   my ($origdate,$offset,$format) = @_;
   #set a default date format
   $format=$format||"yyyymmdd";
   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
 
   #add $offset days worth of seconds to the unix integer date
   #convert the date back to an english string and parse it
   my $newdate=localtime(($origdate+($offset*86400)));
   
   my ($nmonth,$nday,$nyear)=($newdate)=~/\w+\s+(\w{3})\s+(\d+)\s+.+(\d{4})/;
   $nday=sprintf("%02d",$nday);
   $format=~s/[yY]+/$nyear/;
   $format=~s/[mM]+/$month_num{$nmonth}/;
   $format=~s/[dD]+/$nday/;
   return "$format";
}

#return a yyyymmdd datestring from the current date (in yyyymmdd format)
sub curr_date {
   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
   #get the current unix integer time
   #convert the current time to an english string and parse it
   my $currdate=localtime(time());
   my ($cmonth,$cday,$cyear)=($currdate)=~/\w+\s+(\w{3})\s+(\d+)\s+.+(\d{4})/;
   $cday=sprintf("%02d",$cday);
   return "$cyear$month_num{$cmonth}$cday";
}

# return a UNIX integer date from a datestring.  
# Accepts yyyymmdd, and mm/dd/yyyy formats.  hours, mins, secs are not used.
sub unix_date {
   my ($indate)=@_;
   my ($mm,$dd,$yyyy,$timestamp);
   #accept the format mm/dd/yyyy
   if ($indate=~/^\d{2}\/\d{2}\/\d{4}$/) {
      ($mm,$dd,$yyyy)=$indate=~/^(\d{2})\/(\d{2})\/(\d{4})$/;
   }elsif  ($indate=~/^\d{8}$/) {
      #default format yyyymmdd
      ($yyyy,$mm,$dd)=$indate=~/^(\d{4})(\d{2})(\d{2})$/;
   }
   use POSIX;
   $mm--;
   $yyyy-=1900;
   $timestamp = mktime(0,0,0,$dd,$mm,$yyyy,0,0,-1);
   return "$timestamp";
}

sub mail_message {
   my ($aref) = @_;
   my %args=%$aref;

   local $,=undef; # set output field separator
   local $\=undef; # set output record separator

   my $replyto=();
   if ($args{replyto}) {
      $replyto="-r\"$args{replyto}\"";
   }

   my $ccto=();
   if ($args{ccto}) {
      $ccto="-c\"$args{ccto}\"";
   }

   my $bccto=();
   if ($args{bccto}) {
      $bccto="-b\"$args{bccto}\"";
   }

   #print "mailx  $replyto $bccto $ccto -s\"$args{subject}\" $args{to}";

   open (outfile,"| mailx $replyto $bccto $ccto -s\"$args{subject}\" $args{to}");
   print outfile ("$args{message} \n\n");
   close(outfile);

}#sub mail_message

sub logit {
   my ($logaction,$string)=@_;
   return if (!$logaction);
   local $\="\n"; # set output record separator
   #use the ENV{logfile} variable for the file if it is defined.
   $logfile=$ENV{logfile}||(split("/",$0))[-1].".log";

   #quick map to look up the month number
   my %month_num=qw(
   Jan 01
   Feb 02
   Mar 03
   Apr 04
   May 05
   Jun 06
   Jul 07
   Aug 08
   Sep 09
   Oct 10
   Nov 11
   Dec 12
   );
   #split the date string into the separate pieces
   my($cmonth,$cday,$ctime,$cyear)= localtime(time()) =~ /^.{3}\s(.{3})\s(.{2})\s(.{8})\s(.{4})$/;
   $cday=sprintf("%02d",$cday);
   $cdate = "$cyear$month_num{$cmonth}$cday";
   my $datestring="$cdate $ctime";
   
   if (!$string && $logaction !~/^init$|^delete$/i) {
      $string=$logaction;
      $logaction="print";
   }
   my $status;
   switch: for ($logaction) {
      /^init$/i && do {
         $status=open(outfile, ">/tmp/$logfile") || die "Cant open file - (/tmp/$logfile)";
         $status=print outfile "\n$datestring  Logfile Created.";
         close outfile;
         system("chmod 777 /tmp/$logfile");
         last switch;
      };
      /^print$/i && do {
         if (! -f "/tmp/$logfile") {
            system("touch /tmp/$logfile");
            system("chmod 777 /tmp/$logfile");
         }
         open(outfile, ">>/tmp/$logfile") || die "Cant open file - (/tmp/$logfile)";
         $status=print outfile "$datestring $string";
         close outfile;
         last switch;
      };
      /^delete$/i && do {
         $status=1;
         if (-f "/tmp/$logfile") {
            $status=unlink("/tmp/$logfile");
         }
         last switch;
      };
   }#switch
   return ($status);
}#sub logit


1;

__END__
